from os import mkdir
from os.path import expanduser, join
from datetime import datetime
from logging import getLogger, FileHandler, StreamHandler

from cml.shared.errors import NoModelReconstructedError
from cml.shared.parameter import PROTOCOL_LEVEL
from cml.usecases.usecase import Usecase


class KnowledgeSearchUsecase(Usecase):
    def process(self, request):
        source, constr, f_selectr, reconstr, deconstr = self._init_loggers(request)
        reconstr.category = constr.construction_type
        reconstr.free_id = deconstr.knowledge_database.generate_free_ids()
        for learnblock in source.learnblocks:
            for complete_learnblock in constr.construct(learnblock):
                try:
                    reduced_learnblock = f_selectr.select(complete_learnblock)
                    pragmatic_model = reconstr.reconstruct(reduced_learnblock)
                    print(pragmatic_model.uid)
                    print(pragmatic_model.purpose)
                    print(pragmatic_model.subject)
                    print(pragmatic_model.meta)
                    #deconstr.deconstruct(pragmatic_model)
                except ValueError as error:
                    # Can't reduce learnblock
                    # log
                    continue
                except NoModelReconstructedError:
                    continue

    def _init_loggers(self, request):
        log_dir = create_log_dir()
        iteration_log_file = join(log_dir, "iteration.log")
        iteration_handler = FileHandler(iteration_log_file)
        iteration_handler.level = PROTOCOL_LEVEL
        iteration_logger = getLogger("iterationLogger")
        iteration_logger.addHandler(iteration_handler)
        request.data_source.logger = iteration_logger
        request.constructor.logger = iteration_logger
        request.feature_selector.logger = iteration_logger

        if request.stdout:
            iteration_stdout_handler = StreamHandler()
            iteration_stdout_handler.level = PROTOCOL_LEVEL
            iteration_logger.addHandler(iteration_stdout_handler)

        return (request.data_source,
                request.constructor,
                request.feature_selector,
                request.reconstructor,
                request.deconstructor)


def create_log_dir():
    log_dir = join(expanduser('~'),
                   ".cml",
                   datetime.now().strftime("%d%m%Y%H%M%S"))
    mkdir(log_dir)
    return log_dir

